from dataclasses import dataclass

import pandas as pd
from flatten_json import flatten

from sportradar_api import SoccerExtended
from sportradar_api.main import SportradarAPI
from sportradar_api.utils.utils import explode_column, remove_str


@dataclass
class SoccerExtendedPandas(SportradarAPI):
    """Parser to transform the SoccerExtended API response to Pandas DataFrame"""

    api: str = "soccer-extended"

    def __post_init__(self):
        self.soccer_extended = SoccerExtended(api_key=self.api_key, verbose=self.verbose)

    def get_competitions(self) -> pd.DataFrame:
        """Get all available Soccer competitions.

        Returns:
            Pandas DataFrame
        """
        competitions = self.soccer_extended.get_competitions()
        competitions = pd.json_normalize(competitions["competitions"])

        return competitions

    def get_seasons(self) -> pd.DataFrame:
        """Get historical season information for all competitions.

        Returns:
            Pandas DataFrame
        """
        seasons = self.soccer_extended.get_seasons()
        seasons = pd.json_normalize(seasons["seasons"])

        return seasons

    def get_season_matches_statistics(self, season_urn: str) -> pd.DataFrame:
        """Get the players statistics of all matches from a given season.

        Args:
            season_urn: URN of a given season

        Returns:
            Pandas DataFrame
        """
        season_summaries = self.soccer_extended.get_season_summaries(season_urn=season_urn)

        matches_statistics = (
            pd.json_normalize(season_summaries, "summaries")
            .pipe(explode_column, "statistics.totals.competitors", ["sport_event.id", "sport_event.start_time"])
            .pipe(
                explode_column,
                "statistics.totals.competitors.players",
                [
                    "sport_event.id",
                    "sport_event.start_time",
                    "statistics.totals.competitors.id",
                    "statistics.totals.competitors.qualifier",
                ],
            )
            .pipe(
                lambda x: x.set_axis(
                    [remove_str("_".join(col.split(".")[-2:]), ["sport_event_", "statistics_"]) for col in x.columns],
                    axis=1,
                )
            )
        )

        return matches_statistics

    def get_season_competitors(self, season_urn: str) -> pd.DataFrame:
        """Get all teams participating for a given season.

        Args:
            season_urn: URN of a given season

        Returns:
            Pandas DataFrame
        """
        season_competitors = self.soccer_extended.get_season_competitors(season_urn=season_urn)
        season_competitors = pd.json_normalize(season_competitors["season_competitors"])

        return season_competitors.assign(season_urn=season_urn)

    def get_season_competitor_player(self, season_urn: str) -> pd.DataFrame:
        """Get all players profile for a given season

        Args:
            season_urn: URN of a given season

        Returns:
            Pandas DataFrame
        """
        season_competitor_players = self.soccer_extended.get_season_competitor_players(season_urn=season_urn)
        season_competitor_players = (
            pd.json_normalize(season_competitor_players["season_competitor_players"])
            .explode("players")
            .players.apply(pd.Series)
        )

        return season_competitor_players

    def get_player_profile_info(self, player_urn: str) -> pd.DataFrame:
        """Get the basic information from a player profile

        Args:
            player_urn: URN of a given player

        Returns:
            Pandas DataFrame
        """
        player_profile = self.soccer_extended.get_player_profile(player_urn=player_urn)
        player_profile_info = pd.json_normalize(player_profile["player"])

        return player_profile_info.assign(player_id=player_urn)

    def get_player_profile_competitors(self, player_urn: str) -> pd.DataFrame:
        """Get the competitors from a player profile

        Args:
            player_urn: URN of a given player

        Returns:
            Pandas DataFrame
        """
        player_profile = self.soccer_extended.get_player_profile(player_urn=player_urn)
        player_profile_competitors = pd.json_normalize(player_profile["competitors"])

        return player_profile_competitors.assign(player_id=player_urn)

    def get_player_profile_roles(self, player_urn: str) -> pd.DataFrame:
        """Get the roles from a player profile

        Args:
            player_urn: URN of a given player

        Returns:
            Pandas DataFrame
        """
        player_profile = self.soccer_extended.get_player_profile(player_urn=player_urn)
        player_profile_roles = pd.json_normalize([flatten(role, separator=".") for role in player_profile["roles"]])

        return player_profile_roles.assign(player_urn=player_urn)



import logging
import os
from dataclasses import dataclass
from time import sleep
from typing import Optional

import requests
from requests import HTTPError, Response


@dataclass
class SportradarAPI:
    api: str
    api_key: Optional[str] = None
    api_access_level: str = "trial"
    api_version: str = "v4"
    api_language_code: str = "en"
    api_format: str = "json"
    timeout: int = 120
    sleep_time: int = 1.2
    verbose: bool = True

    def __post_init__(self):
        self._set_logger()
        self._set_api_key()

    def _set_logger(self):
        if self.verbose:
            logging.basicConfig(
                level=logging.INFO,
                format="[%(asctime)s] [SportradarAPI] %(message)s",
                datefmt="%Y-%m-%d %H:%M:%S",
            )
        self.log = logging.getLogger()

    def _set_api_key(self):
        if not self.api_key:
            self.api_key = os.getenv("SPORTRADAR_API_KEY")
            assert self.api_key, "Please provide `api_key` parameter or set SPORTRADAR_API_KEY environment variable"
            self.log.info(f'Loaded SPORTRADAR_API_KEY: {self.api_key[:4].ljust(len(self.api_key), "*")}')
        else:
            self.log.info(f'Loaded `api_key` parameter: {self.api_key[:4].ljust(len(self.api_key),"*")}')

    def _call_endpoint(self, endpoint: str, key: Optional[str] = None) -> dict:
        response = self._make_request(endpoint=endpoint)
        content = response.json()

        request_results = response.headers.get("X-Result")
        request_max_results = response.headers.get("X-Max-Results")
        self.log.info(f"[{endpoint}] Headers: {response.headers}")

        if request_results is None or request_max_results is None:
            self.log.info(f"[{endpoint}] Fetching complete!")
            return content

        request_results = int(request_results)
        request_max_results = int(request_max_results)

        if request_max_results - request_results <= 0:
            self.log.info(f"[{endpoint}] Total records: {len(content.get(key))}")
            self.log.info(f"[{endpoint}] Fetching complete!")
            return content

        for offset in range(request_results, request_max_results, request_results):
            response = self._make_request(endpoint=endpoint, offset=offset, limit=request_results)
            content[key].extend(response.json().get(key))

        self.log.info(f"[{endpoint}] Parsed records: {len(content.get(key))}")
        self.log.info(f"[{endpoint}] Fetching complete!")
        return content

    def _make_request(self, endpoint: str, offset: Optional[int] = None, limit: Optional[int] = None) -> Response:
        sleep(self.sleep_time)

        url = (
            "https://api.sportradar.us"
            f"/{self.api}"
            f"/{self.api_access_level}"
            f"/{self.api_version}"
            f"/{self.api_language_code}"
            f"/{endpoint}"
            f".{self.api_format}"
        )

        self.log.info(f"[{endpoint}] Calling endpoint...")
        self.log.info(f"[{endpoint}] Parameters: {offset=} {limit=}")

        response = requests.get(
            url,
            timeout=self.timeout,
            params={"api_key": self.api_key, "offset": offset, "limit": limit},
        )

        self.log.info(f"[{endpoint}] Response Status Code: {response.status_code}")
        self.log.info(
            f"[{endpoint}] Updated API Key Quota: "
            f"{response.headers.get('X-Plan-Quota-Current')}"
            f"/{response.headers.get('X-Plan-Quota-Allotted')}"
        )

        if response.status_code == 200:
            return response

        elif 400 <= response.status_code < 500:
            raise HTTPError(f"{response.status_code} Client Error: {response.reason} for url: {url}", response=response)

        elif 500 <= response.status_code < 600:
            raise HTTPError(f"{response.status_code} Server Error: {response.reason} for url: {url}", response=response)



from dataclasses import dataclass

from sportradar_api.main import SportradarAPI


@dataclass
class SoccerExtended(SportradarAPI):
    """Wrapper to interact with Sportradar SoccerExtended API"""

    api: str = "soccer-extended"

    def get_competitions(self) -> dict:
        """Get all available Soccer competitions.

        Returns:
            API response
        """
        return self._call_endpoint(endpoint="competitions", key="competitions")

    def get_seasons(self) -> dict:
        """Get historical season information for all competitions.

        Returns:
            API response
        """
        return self._call_endpoint(endpoint="seasons", key="seasons")

    def get_season_summaries(self, season_urn: str) -> dict:
        """Get the summaries for all sport events in a season (any status). Provides information for all matches from a
        given season including scoring and statistics at the match level.

        Args:
            season_urn:

        Returns:
            API response
        """
        return self._call_endpoint(endpoint=f"seasons/{season_urn}/summaries", key="summaries")

    def get_season_competitors(self, season_urn: str) -> dict:
        """Get all teams participating for a given season.

        Args:
            season_urn: URN of a given season

        Returns:
            API response
        """
        return self._call_endpoint(endpoint=f"seasons/{season_urn}/competitors", key="season_competitors")

    def get_season_competitor_players(self, season_urn: str) -> dict:
        """Get the competitors and their players in a season. Provides player roster information for every team from a
         given season.

        Args:
            season_urn: URN of a given season

        Returns:
            API response
        """
        return self._call_endpoint(endpoint=f"seasons/{season_urn}/competitor_players", key="season_competitor_players")

    def get_player_profile(self, player_urn: str) -> dict:
        """Get the player profile for the given urn. Provides player information, including current and historical team
        membership info.

        Args:
            player_urn: URN of a given player

        Returns:
            API response
        """

        return self._call_endpoint(endpoint=f"players/{player_urn}/profile")



from typing import Optional

import pandas as pd


def remove_str(text: str, strings_to_remove: list) -> str:
    for string in strings_to_remove:
        text = text.replace(string, "")

    return text


def explode_column(
    df_to_explode: pd.DataFrame, col_to_explode: str, cols_to_keep: Optional[list] = None
) -> pd.DataFrame:
    df = df_to_explode.copy()
    if cols_to_keep:
        df = df.loc[:, cols_to_keep + [col_to_explode]]

    df = df.explode(col_to_explode)

    normalized = pd.json_normalize(df[col_to_explode]).add_prefix(f"{col_to_explode}.")
    normalized.index = df.index

    df = pd.concat([df, normalized], axis=1)
    df = df.drop(columns=col_to_explode)

    return df


def remove_cols_str(df: pd.DataFrame, str_list: list) -> pd.DataFrame:
    df.columns = [remove_str(col, str_list) for col in df.columns]
    return df



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sportradar_api', 'sportradar_api.soccer_extended', 'sportradar_api.utils']

package_data = \
{'': ['*']}

install_requires = \
['flatten-json==0.1.13', 'pandas==1.5.3', 'requests==2.28.2']

setup_kwargs = {
    'name': 'sportradar-api',
    'version': '0.2.2',
    'description': 'Lightweight wrapper for Sportradar API',
    'long_description': "# Sportradar API\nLightweight wrapper for [Sportradar API](https://developer.sportradar.com/docs/read/Home)\n\n## Set up\n1. Register on [Sportradar Developer](https://developer.sportradar.com/member/register)\n2. Generate an API Key\n\n## Installation\n````bash\npip install sportradar-api\n````\n\n## Usage\n\n````python\nfrom sportradar_api import SoccerExtended\n\nsportradar = SoccerExtended(api_key='SPORTRADAR_API_KEY')\n````",
    'author': 'Felipe Allegretti',
    'author_email': 'felipe@allegretti.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://felipeall.github.io/sportradar-api',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
